/****************************************************************************
  Interfacing PIC18F46K22 microcontroller with ILI9341 TFT 
    display (240x320 pixel).
  Graphics test example.
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 64MHz
  Configuration words: CONFIG1H = 0x0028
                       CONFIG2L = 0x0018
                       CONFIG2H = 0x003C
                       CONFIG3H = 0x0037
                       CONFIG4L = 0x0081
                       CONFIG5L = 0x000F
                       CONFIG5H = 0x00C0
                       CONFIG6L = 0x000F
                       CONFIG6H = 0x00E0
                       CONFIG7L = 0x000F
                       CONFIG7H = 0x0040
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

*****************************************************************************
  This is our GFX example for the Adafruit ILI9341 Breakout and Shield
  ----> http://www.adafruit.com/products/1651

  Check out the links above for our tutorials and wiring diagrams
  These displays use SPI to communicate, 4 or 5 pins are required to
  interface (RST is optional)
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
*****************************************************************************/

// define ILI9341 TFT module pin connections
//   RST (optional), CS and DC
#define TFT_CS        RD0_bit
#define TFT_RST       RD1_bit
#define TFT_DC        RD2_bit
#define TFT_CS_DIR    TRISD0_bit
#define TFT_RST_DIR   TRISD1_bit
#define TFT_DC_DIR    TRISD2_bit

#include "ILI9341.c"       // include ILI9341 TFT display driver
#include "GFX_Library.c"   // include graphics library source code

void testFillScreen() {
  display_fillScreen(ILI9341_BLACK);
  display_fillScreen(ILI9341_RED);
  display_fillScreen(ILI9341_GREEN);
  display_fillScreen(ILI9341_BLUE);
  display_fillScreen(ILI9341_BLACK);
}

void testText() {
  display_fillScreen(ILI9341_BLACK);
  display_setCursor(0, 0);
  display_setTextColor(ILI9341_WHITE, ILI9341_WHITE);  // set text color to white with transparant background
  display_setTextSize(1);
  display_puts("Hello World!\r\n");
  display_setTextColor(ILI9341_YELLOW, ILI9341_YELLOW);
  display_setTextSize(2);
  display_printf("%7.2f\r\n", 1234.56);
  display_setTextColor(ILI9341_RED, ILI9341_RED);
  display_setTextSize(3);
  display_printf("%LX\r\n", 0xDEADBEEF);
  display_puts("\r\n");    // start a new line
  display_setTextColor(ILI9341_GREEN, ILI9341_GREEN);
  display_setTextSize(5);
  display_puts("Groop\r\n");
  display_setTextSize(2);
  display_puts("I implore thee,\r\n");
  display_setTextSize(1);
  display_puts("my foonting turlingdromes.\r\n");
  display_puts("And hooptiously drangle me\r\n");
  display_puts("with crinkly bindlewurdles,\r\n");
  display_puts("Or I will rend thee\r\n");
  display_puts("in the gobberwarts\r\n");
  display_puts("with my blurglecruncheon,\r\n");
  display_puts("see if I don't!\r\n");
}

void testLines(uint16_t color) {
  int16_t x1, y1, x2, y2,
          w = display_width,
          h = display_height;

  display_fillScreen(ILI9341_BLACK);

  x1 = y1 = 0;
  y2    = h - 1;

  for(x2=0; x2<w; x2+=6) display_drawLine(x1, y1, x2, y2, color);
  x2    = w - 1;
  for(y2=0; y2<h; y2+=6) display_drawLine(x1, y1, x2, y2, color);

  display_fillScreen(ILI9341_BLACK);

  x1    = w - 1;
  y1    = 0;
  y2    = h - 1;

  for(x2=0; x2<w; x2+=6) display_drawLine(x1, y1, x2, y2, color);
  x2    = 0;
  for(y2=0; y2<h; y2+=6) display_drawLine(x1, y1, x2, y2, color);

  display_fillScreen(ILI9341_BLACK);

  x1    = 0;
  y1    = h - 1;
  y2    = 0;

  for(x2=0; x2<w; x2+=6) display_drawLine(x1, y1, x2, y2, color);
  x2    = w - 1;
  for(y2=0; y2<h; y2+=6) display_drawLine(x1, y1, x2, y2, color);

  display_fillScreen(ILI9341_BLACK);

  x1    = w - 1;
  y1    = h - 1;
  y2    = 0;

  for(x2=0; x2<w; x2+=6) display_drawLine(x1, y1, x2, y2, color);
  x2    = 0;
  for(y2=0; y2<h; y2+=6) display_drawLine(x1, y1, x2, y2, color);
}

void testFastLines(uint16_t color1, uint16_t color2) {
  int16_t x, y, w = display_width, h = display_height;

  display_fillScreen(ILI9341_BLACK);

  for(y=0; y<h; y+=5) display_drawHLine(0, y, w, color1);
  for(x=0; x<w; x+=5) display_drawVLine(x, 0, h, color2);
}

void testRects(uint16_t color) {
  int16_t n, i, i2,
          cx = display_width  / 2,
          cy = display_height / 2;

  display_fillScreen(ILI9341_BLACK);
  n = min(display_width, display_height);

  for(i=2; i<n; i+=6) {
    i2 = i / 2;
    display_drawRect(cx-i2, cy-i2, i, i, color);
  }
}

void testFilledRects(uint16_t color1, uint16_t color2) {
  int16_t n, i, i2,
          cx = display_width  / 2 - 1,
          cy = display_height / 2 - 1;

  display_fillScreen(ILI9341_BLACK);
  n = min(display_width, display_height);
  for(i=n; i>0; i-=6) {
    i2 = i / 2 - 1;

    display_fillRect(cx-i2, cy-i2, i, i, color1);

    // Outlines are not included in timing results
    display_drawRect(cx-i2, cy-i2, i, i, color2);
  }
}

void testFilledCircles(uint8_t radius, uint16_t color) {
  int16_t x, y, w = display_width, h = display_height, r2 = radius * 2;

  display_fillScreen(ILI9341_BLACK);

  for(x=radius; x<w; x+=r2) {
    for(y=radius; y<h; y+=r2) {
      display_fillCircle(x, y, radius, color);
    }
  }
}

void testCircles(uint8_t radius, uint16_t color) {
  int16_t x, y, r2 = radius * 2,
          w = display_width  + radius,
          h = display_height + radius;

  for(x=0; x<w; x+=r2) {
    for(y=0; y<h; y+=r2) {
      display_drawCircle(x, y, radius, color);
    }
  }
}

void testTriangles() {
  int16_t n, i, cx = display_width  / 2 - 1,
          cy = display_height / 2 - 1;
  uint16_t color = 0xF600;
  display_fillScreen(ILI9341_BLACK);
  n = min(cx, cy);

  for(i=0; i<n; i+=5) {
    display_drawTriangle(
      cx    , cy - i, // peak
      cx - i, cy + i, // bottom left
      cx + i, cy + i, // bottom right
      color);
    color += 100;
  }
}

void testFilledTriangles() {
  int16_t i, cx = display_width  / 2 - 1,
             cy = display_height / 2 - 1;

  display_fillScreen(ILI9341_BLACK);

  for(i=min(cx,cy); i>10; i-=5) {

    display_fillTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i,
      display_color565(0, i*2, i*2));

    display_drawTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i,
      display_color565(i*2, i*2, 0));

  }
}

void testRoundRects() {
  int16_t w, i, i2,
          cx = display_width  / 2 - 1,
          cy = display_height / 2 - 1;

  display_fillScreen(ILI9341_BLACK);
  w = min(display_width, display_height);

  for(i=0; i<w; i+=6) {
    i2 = i / 2;
    display_drawRoundRect(cx-i2, cy-i2, i, i, i/8, display_color565(i, 0, 0));
  }
}

void testFilledRoundRects() {
  int16_t i, i2,
          cx = display_width  / 2 - 1,
          cy = display_height / 2 - 1;

  display_fillScreen(ILI9341_BLACK);

  for(i=min(display_width, display_height); i>20; i-=6) {
    i2 = i / 2;
    display_fillRoundRect(cx-i2, cy-i2, i, i, i/8, display_color565(0, i, 0));
  }
}

// main function
void main() {

  OSCCON = 0x70;   // set internal oscillator to 16MHz
  PLLEN_bit = 1;   // enable 4xPLL (clock frequency becomes 16 x 4 = 64MHz)
  ANSELC = 0;      // configure all PORTC pins as digital
  ANSELD = 0;      // configure all PORTD pins as digital

  // initialize SPI1 module
  SPI1_Init();
  
  // initialize ILI9341 TFT
  tft_begin();
  // init done!

  delay_ms(10);
  testFillScreen();
  delay_ms(500);

  testText();
  delay_ms(3000);

  testLines(ILI9341_CYAN);
  delay_ms(500);

  testFastLines(ILI9341_RED, ILI9341_BLUE);
  delay_ms(500);

  testRects(ILI9341_GREEN);
  delay_ms(500);

  testFilledRects(ILI9341_YELLOW, ILI9341_MAGENTA);
  delay_ms(500);

  testFilledCircles(10, ILI9341_MAGENTA);

  testCircles(10, ILI9341_WHITE);
  delay_ms(500);

  testTriangles();
  delay_ms(500);

  testFilledTriangles();
  delay_ms(500);

  testRoundRects();
  delay_ms(500);

  testFilledRoundRects();
  delay_ms(500);

  while(1) {
    uint8_t rotation;
    for(rotation=0; rotation<4; rotation++) {
      display_setRotation(rotation);
      testText();
      delay_ms(1000);
    }

  }
}

// end of code.